/*
 * (c) Copyright 1999-2007 PaperCut Software Pty. Ltd.
 * $Id: ExternalCreditSourceProvider.java 5498 2008-01-14 03:47:45Z priyanka $
 */
package biz.papercut.pcng.ext.paymentgateway;

import java.util.Properties;

/**
 * This interface an external credit source provider.  The steps to implement a new "external credit source"
 * provider is as follows:
 *    * Write a java class that implements this interface.
 *    
 *    * Add the class (and any associated infrastructure classes) to a standard JAR file.
 *    
 *    * Place the JAR file in [app-path]/server/lib-ext
 *    
 *    * Modify the payment gateway configuration file so the full class name is listed as the custom source. 
 *              e.g. edu.mydomain.mypackage.MyClass
 *              
 *    * Restart the Application Server and test.
 *    
 * The developer should take care in all methods to avoid throwing unnecessary Exceptions.
 * 
 * During development, this interface can be referenced by including the file:
 *      [app-path]/server/lib-ext/ext-payment-gateway.jar
 * on your project's classpath.
 * 
 * On the end-user side, an "external credit source" works as follows:
 * 
 *    * The user logs into the PaperCut end-user web interface.
 *  
 *    * They click on the "Add Credit" link
 *  
 *    * They select the amount to add and click the "Add" button.
 *  
 *    * The payment gateway calls the withdrawCredit() method.
 *  
 *    * Your implementation of this interface performs the adjustment in the external system and returns a status. The
 *      method either returns "success" or "failure" and as associated message providing feedback to the user.  For 
 *      example, "Not enough funds in your external account...".
 * 
 * @author chris
 */
public interface ExternalCreditSourceProvider {
    
    /**
     * @return Return the name the write has assigned to the source.  This is used to display the name of the source
     * in the payment gateway web interface.  For example, return "University Online Student Account".
     */
    String getSourceName();
    
    /**
     * This method is call after instancing to pass in standard Java properties object.  The Properties object
     * is constructed by read the payment gateway configuration file.  This is provided so you can use the config
     * file to store/set your own configuration options.
     * @param properties The properties read from the configuration file.
     */
    void setConfig(Properties properties);
    
    /**
     * Main method used to withdraw credit from the external source.
     * @param username The username of the user associated with the withdraw (always set).
     * @param fullName The fullname of the user associated with the withdraw (can be null/empty).
     * @params cardNumber The card number assoicated with the user (can be null/empty).
     * @param amount The amount to withdraw from the external source.
     * @param transactionComment A comment associated with the transactions.
     * @return A RequestResponse object used to return status (success/failure, and error message).
     */
    RequestResponse withdrawCredit(String username, 
            String fullName, 
            String cardNumber, 
            double amount, 
            String transactionComment);
    
    /**
     * This method should return the amount of available funds in the external credit source, or
     * <code>null</code> if this is not appropriate.  This value will be used to provide some information
     * to the user on the "Add Credit" or transfer page.
     * @param username The username of the user (always set).
     * @param fullName The fullname of the user (can be null/empty).
     * @param cardNumber The card number associated with the user (can be null/empty).
     * @return A Double representing the available credit, or null if not applicable/known.
     */
    Double availableCredit(String username, String fullName, String cardNumber);
    
    /**
     * A transfer object for a request response.
     * 
     * @author chris
     */
    class RequestResponse {
        
        private boolean _success;
        private String _message;
        
        /**
         * Default constructor.
         * @param success True if the request was allowed.
         * @param message A message associated with the request.  Normally leave blank if allowed.
         */
        public RequestResponse(boolean success, String message) {
            _success = success;
            _message = message;
        }

        /**
         * @return the success status (true for success)
         */
        public boolean isSuccess() {
            return _success;
        }

        /**
         * @param success the success status to set (true for success)
         */
        public void setSuccess(boolean success) {
            _success = success;
        }
        
        /**
         * @return The status message to return (viewed by users).
         */
        public String getMessage() {
            return _message;
        }

        /**
         * @param msg The status message to set.
         */
        public void setMessage(String msg) {
            _message = msg;
        }
        
    }
    

}
